syntax = "proto3";

package xray.common.log;
option csharp_namespace = "Xray.Common.Log";
option go_package = "github.com/xtls/xray-core/common/log";
option java_package = "com.xray.common.log";
option java_multiple_files = true;

enum Severity {
  Unknown = 0;
  Error = 1;
  Warning = 2;
  Info = 3;
  Debug = 4;
}
