syntax = "proto3";

package xray.transport.internet.headers.wireguard;
option csharp_namespace = "Xray.Transport.Internet.Headers.Wireguard";
option go_package = "github.com/xtls/xray-core/transport/internet/headers/wireguard";
option java_package = "com.xray.transport.internet.headers.wireguard";
option java_multiple_files = true;

message WireguardConfig {}
