syntax = "proto3";

package xray.transport.internet.headers.utp;
option csharp_namespace = "Xray.Transport.Internet.Headers.Utp";
option go_package = "github.com/xtls/xray-core/transport/internet/headers/utp";
option java_package = "com.xray.transport.internet.headers.utp";
option java_multiple_files = true;

message Config {
  uint32 version = 1;
}
